<?php

namespace Drupal\patreon_user\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\Config;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Drupal\patreon\PatreonServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Provides a 'PatreonUserBlock' block.
 *
 * @Block(
 *  id = "patreon_user_block",
 *  admin_label = @Translation("Patreon user block"),
 * )
 */
class PatreonUserBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal\patreon_user\PatreonUserService definition.
   *
   * @var \Drupal\patreon\PatreonServiceInterface
   */
  protected PatreonServiceInterface $patreonUserApi;

  /**
   * The module configuration.
   *
   * @var \Drupal\Core\Config\Config
   *   The configuration object.
   */
  protected Config $config;

  /**
   * Constructs a new PatreonUserBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\patreon\PatreonServiceInterface $patreon_user_api
   *   The API User Service.
   * @param \Drupal\Core\Config\Config $config
   *   The module config.
   */
  public function __construct(
        array $configuration,
        string $plugin_id,
        array $plugin_definition,
        PatreonServiceInterface $patreon_user_api,
        Config $config
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->patreonUserApi = $patreon_user_api;
    $this->config = $config;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('patreon_user.api'),
      $container->get('config.factory')->getEditable('patreon.settings')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    if ($account->isAnonymous()) {
      return AccessResult::allowed()
        ->addCacheContexts(['user.roles:anonymous']);
    }
    return AccessResult::forbidden();
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $key = $this->config->get('patreon_client_id');
    if ($path = $this->patreonUserApi->getReturnPath()) {
      $url = Url::fromRoute($path['route_name'], $path['route_parameters']);
      $return = $url->toString();
      $context = 'user.roles';
    }
    else {
      $return = '';
      $context = 'url.path';
    }
    $url = $this->patreonUserApi->authoriseAccount($key, [
      'identity',
      'identity[email]',
      'identity.memberships',
      'campaigns.members',
    ], $return, FALSE);
    $build = [];
    $build['patreon_user_block'] = [
      '#title' => $this->t('Login via Patreon'),
      '#type' => 'link',
      '#url' => $url,
      '#cache' => [
        'contexts' => [$context],
      ],
    ];

    return $build;
  }

}
